from collections import namedtuple
from copy import deepcopy
from _function import is_seq, is_math, is_iter
from random import random
import csv

__all__ = ['Matrix']

class Matrix(object):

    dims = namedtuple('Matrix', ['Ln', 'Col'])

    def __init__(self, table=None, check=True):

        self._miss_value = None

        if is_iter(table):
            self.__init_unknow_type(table, check)
                 
        elif isinstance(table, Matrix):
            self._matrix = deepcopy(table)
            self._dim = deepcopy(table._dim)

        elif table is None:
            self._matrix = list()
            self._dim = Matrix.dims(0, 0)
            
        else:
            raise TypeError('can not transform this object to DaPy.Matrix'+\
                            ', which expects iterables in iterable.')

    @property
    def data(self):
        return self._matrix

    @property
    def shape(self):
        return self._dim

    @property
    def T(self):
        new_ = list()
        for j in range(self._dim.Col):
            new_.append([record[j] for record in self._matrix])
        return Matrix(new_, False)
    
    def __repr__(self):
        temporary_series = [list()] * self._dim.Col
        if self._dim.Ln > 20:
            temporary_Frame = self._matrix[:10]
            temporary_Frame.extend(self._matrix[-10:])
        else:
            temporary_Frame = self._matrix
            
        for line in temporary_Frame:
            for i, value in enumerate(line):
                temporary_series[i].append(str(value))
        column_size = [len(max(col, key=len)) for col in temporary_series]

        frame = 'matrix(['

        if self._dim.Ln > 20:
            for i, item in enumerate(temporary_Frame[:10]):
                if i != 0:
                    line = '       ['
                for i, value in enumerate(item):
                    line += str(value).center(column_size[i]) + ' '
                frame += line + ']\n'
            frame += ('.. Omit %d Ln ..'%(self._dim.Ln - 20)).center(len(line)) + '\n'
            for item in temporary_Frame[-10:]:
                frame += '       ['
                for i, value in enumerate(item):
                    frame += str(value).center(column_size[i]) + ' '
                frame += ']\n'
                
        else:
            for i, item in enumerate(temporary_Frame):
                if i != 0:
                    frame += '       ['
                for i, value in enumerate(item):
                    frame += str(value).center(column_size[i]) + ' '
                frame += ']\n'
        return frame[:-1] +  ')'
    
    def __contains__(self, e):
        if isinstance(e, list):
            for record in self._matrix:
                if record == e:
                    return True
        return False

    def __len__(self):
        return self._dim.Ln

    def __eq__(self, other):
        if isinstance(other, (Frame, SeriesSet, Matrix)):
            if self._dim == other._dim:
                return True
        return False

    def __getitem__(self, pos):
        return self._matrix[pos]
    
    def __iter__(self):
        for i in xrange(self._dim.Ln):
            yield self._matrix[i]

    def __sum__(self, other):
        return sum([sum(record) for record in self._matrix])
    
    def __abs__(self):
        new_ = [0] * self._dim.Ln
        for i, line in enumerate(self._matrix):
            new_[i] = [abs(value) for value in line]
        return Matrix(new_, False)

    def __add__(self, other):
        new_ = [0] * self._dim.Ln
        if is_math(other):
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] + other\
                           for j in range(self._dim.Col)]

        elif hasattr(other, 'shape'):
            if not self.shape == other.shape:
                raise ValueError('operands could not be broadcast'+\
                                 'together with shapes '+\
                                 '(%d,%d) '%self._dim+\
                                 '(%d,%d)'%other._dim)
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] + other[i][j]\
                           for j in range(self._dim.Col)]
        else:
            raise TypeError("'+' operation expects the type of"+\
                            "number or an array-like object which has "+\
                            "attribute `shape`")

        return Matrix(new_, False)

    def __radd__(self, other):
        return self.__add__(other)
        
    def __sub__(self, other):
        new_ = [0] * self._dim.Ln
        if is_math(other):
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] - other\
                           for j in range(self._dim.Col)]

        elif hasattr(other, 'shape'):
            if not self.shape == other.shape:
                raise ValueError('operands could not be broadcast'+\
                                 'together with shapes '+\
                                 '(%d,%d) '%self._dim+\
                                 '(%d,%d)'%other._dim)
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] - other[i][j]\
                           for j in range(self._dim.Col)]
        else:
            raise TypeError("'-' operation expects the type of"+\
                            "number or an array-like object which has "+\
                            "attribute `shape`")

        return Matrix(new_, False)

    def __rsub__(self, other):
        new_ = [0] * self._dim.Ln
        if is_math(other):
            for i in range(self._dim.Ln):
                new_[i] = [other - self._matrix[i][j]\
                           for j in range(self._dim.Col)]

        elif hasattr(other, 'shape'):
            if not self.shape == other.shape:
                raise ValueError('operands could not be broadcast'+\
                                 'together with shapes '+\
                                 '(%d,%d) '%self._dim+\
                                 '(%d,%d)'%other._dim)
            for i in range(self._dim.Ln):
                new_[i] = [other[i][j] - self._matrix[i][j]\
                           for j in range(self._dim.Col)]
        else:
            raise TypeError("'-' operation expects the type of"+\
                            "number or an array-like object which has "+\
                            "attribute `shape`")

        return Matrix(new_, False)
                
    def __mul__(self, other):
        new_ = [0] * self._dim.Ln
        if is_math(other):
            for i in range(self._dim.Ln):
                new_[i] = [other * self._matrix[i][j]\
                           for j in range(self._dim.Col)]

        elif hasattr(other, 'shape'):
            if not self.shape == other.shape:
                raise ValueError('operands could not be broadcast'+\
                                 'together with shapes '+\
                                 '(%d,%d) '%self._dim+\
                                 '(%d,%d)'%other._dim)
            for i in range(self._dim.Ln):
                new_[i] = [other[i][j] * self._matrix[i][j]\
                           for j in range(self._dim.Col)]
        else:
            raise TypeError("'*' operation expects the type of"+\
                            "number or an array-like object which has "+\
                            "attribute `shape`")

        return Matrix(new_, False)

    def __rmul__(self, other):
        return self.__mul__(other)
                
    def __div__(self, other):
        new_ = [0] * self._dim.Ln
        if is_math(other):
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] / other\
                           for j in range(self._dim.Col)]

        elif hasattr(other, 'shape'):
            if not self.shape == other.shape:
                raise ValueError('operands could not be broadcast'+\
                                 'together with shapes '+\
                                 '(%d,%d) '%self._dim+\
                                 '(%d,%d)'%other._dim)
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] / other[i][j]\
                           for j in range(self._dim.Col)]
        else:
            raise TypeError("'/' operation expects the type of"+\
                            "number or an array-like object which has "+\
                            "attribute `shape`")

        return Matrix(new_, False)

    def __rdiv__(self, other):
        new_ = [0] * self._dim.Ln
        if is_math(other):
            for i in range(self._dim.Ln):
                new_[i] = [other / self._matrix[i][j]\
                           for j in range(self._dim.Col)]

        elif hasattr(other, 'shape'):
            if not self.shape == other.shape:
                raise ValueError('operands could not be broadcast'+\
                                 'together with shapes '+\
                                 '(%d,%d) '%self._dim+\
                                 '(%d,%d)'%other._dim)
            for i in range(self._dim.Ln):
                new_[i] = [other[i][j] / self._matrix[i][j]\
                           for j in range(self._dim.Col)]
        else:
            raise TypeError("'/' operation expects the type of"+\
                            "number or an array-like object which has "+\
                            "attribute `shape`")
        return Matrix(new_, False)

    def __pow__(self, other):
        new_ = [0] * self._dim.Ln
        if is_math(other):
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] ** other\
                           for j in range(self._dim.Col)]

        elif hasattr(other, 'shape'):
            if not self.shape == other.shape:
                raise ValueError('operands could not be broadcast'+\
                                 'together with shapes '+\
                                 '(%d,%d) '%self._dim+\
                                 '(%d,%d)'%other._dim)
            for i in range(self._dim.Ln):
                new_[i] = [self._matrix[i][j] ** other[i][j]\
                           for j in range(self._dim.Col)]
        else:
            raise TypeError("'**' operation expects the type of"+\
                            "number or an array-like object which has "+\
                            "attribute `shape`")

        return Matrix(new_, False)

    def dot(self, other):
        if hasattr(other, 'shape'):
            if self.shape[1] != other.shape[0]:
                raise ValueError('shapes (%d, %d)'%self._dim +\
                                 ' and (%d, %d) not aligned.'%other._dim)
            col_size_1 = self._dim.Col
            col_size_2 = other.shape[1]
            
        elif isinstance(other, (list, tuple, deque)):
            other = Matrix(other)
            if self._dim.Col != other.shape:
                raise ValueError('shapes (%d, %d)'%(self._dim)+\
                                 ' and (%d, %d) not aligned.'%other.shape)
            col_size_1 = self._dim.Col
            col_size_2 = other._dim.Col
            
        else:
            raise TypeError('unsupported operation dot, with type'+\
                            '< Matrix > and '%str(type(self._matrix)) +\
                            '%s and '%str(type(other)))
        
        new_ = [[0]*col_size_2 for k in range(self._dim.Ln)]
        for i in range(self._dim.Ln):
            for pos in range(col_size_2):
                sumup = 0
                for j in range(col_size_1):
                    sumup += self._matrix[i][j]*other[j][pos]
                new_[i][pos] = sumup
        return Matrix(new_, False)

    def __init_unknow_type(self, table, check):
        try:
            dim_col, dim_ln = len(table[0]), len(table)
        except TypeError:
            self._dim = Matrix.dims(len(table), 1)
            self._matrix = [[float(v)] for v in table]
            return
            
        if check:
            self._matrix = [0] * dim_ln
            for i, record in enumerate(table):
                if len(record) != dim_col:
                    raise IndexError("the No.%d record doesn't "%i+\
                                     "have same dimensions.")
                for value in record:
                    if not is_math(value):
                        raise ValueError('value %s in the No.%d'%(value, i)+\
                                        " record is not a number")
                self._matrix[i] = list(record)
        else:
            self._matrix = table
        self._dim = Matrix.dims(len(table), dim_col)

    def make(self, Ln, Col, element=0):
        if not (isinstance(Ln, int) and isinstance(Col, int) and\
                is_math(element)):
            raise TypeError("arguments 'Ln' and 'Col' expect <int> type,"+\
                            "'element' expects <int> or <float> types.")
        self._matrix = [0] * Ln
        element = float(element)
        for i in range(Ln):
            self._matrix[i] = [element for j in range(Col)]
        self._dim = Matrix.dims(Ln, Col)

    def make_random(self, Ln, Col, type_int=False):
        if not (isinstance(Ln, int) and isinstance(Col, int)):
            raise TypeError("arguments `Ln` and `Col` expect <int> type,")
        if not isinstance(type_int, (bool, tuple)):
            raise TypeError("argutments `type_int` expects `False` symbol"+\
                            " or a tuple-like.")

        self._matrix = [0] * Ln
        if type_int:
            for i in range(Ln):
                self._matrix[i] = [randint(*type_int)] * Col
        else:
            for i in range(Ln):
                self._matrix[i] = [random()] * Col
        self._dim = Matrix.dims(Ln, Col)
            
    def read_text(self, addr, first_line=1, sep=','):
        with open(addr, 'r') as f:
            reader = csv.reader(f, delimiter=sep)
            self._matrix = list()
            for m, record in enumerate(reader):
                if m >= first_line - 1:
                    break
            for m, record in enumerate(reader):
                self._matrix.append(map(float, record))

        col = len(max(self._matrix, key=len))
        def has_size(record):
            if len(record) != col:
                return False
            return True

        if not all(map(has_size, self._matrix)):
            raise ValueError('records in your data do not have the same lenth.')
        self._dim = Matrix.dims(len(self._matrix), col)

    def shuffles(self):
        shuffle(self._matrix)
